import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { PartnerDivision } from '../models/partner-division';
import { PartnerDivisionPagination } from '../models/partner-division-pagination';
import { PartnerDivisionService } from '../services/partner-division.service';

@Injectable({
  providedIn: 'root',
})
export class PartnerDivisionResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _partnerDivisionService: PartnerDivisionService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<PartnerDivision | undefined> {
    return this._partnerDivisionService
      .getPartnerDivisionById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class PartnerDivisionsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _partnerDivisionService: PartnerDivisionService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: PartnerDivisionPagination;
    partnerDivisions: PartnerDivision[];
  }> {
    return this._partnerDivisionService.getPartnerDivisions();
  }
}
